﻿using System;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Robware.Api.Auth.Models;
using Robware.Auth.Users;

namespace Robware.Api.Auth.Controllers {
	[ApiController]
	[Route("[controller]")]
	public class UserController : ControllerBase {
		private readonly ILogger<UserController> _logger;
		private readonly IAuthenticator _authenticator;

		public UserController(ILogger<UserController> logger, IAuthenticator authenticator) {
			_logger = logger;
			_authenticator = authenticator;
		}

		[HttpPost(nameof(Authenticate))]
		public async Task<ActionResult<User>> Authenticate(LoginRequest request) {
			var (result, user) = await _authenticator.Authenticate(request.Username, request.Password);
			switch (result) {
				case AuthenticationResult.Success:
					return user;
				case AuthenticationResult.NotFound:
					return NotFound();
				case AuthenticationResult.IncorrectPassword:
					return Unauthorized();
				default:
					throw new ArgumentOutOfRangeException();
			}
		}

		[HttpPatch(nameof(UpdatePassword))]
		public async Task<ActionResult> UpdatePassword(string username, string oldPassword, string newPassword) {
			var result = await _authenticator.UpdateUserPassword(username, oldPassword, newPassword);
			return result == AuthenticationResult.Success ? (ActionResult)NoContent() : BadRequest();
		}
	}
}
