using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using MongoDB.Driver;
using Robware.Api.Auth.Authentication;
using Robware.Auth.API;
using Robware.Auth.Users;
using Robware.Data;

namespace Robware.Api.Auth {
	public class Startup {
		public Startup(IConfiguration configuration) {
			Configuration = configuration;
		}

		public IConfiguration Configuration { get; }

		// This method gets called by the runtime. Use this method to add services to the container.
		public void ConfigureServices(IServiceCollection services) {
			services.AddControllers();

			services.AddSingleton<ICryptographyProvider, CryptographyProvider>()
			        .AddSingleton<IAuthenticator, Authenticator>()
					.AddSingleton<IMongoDatabase>(SetupMongoDatabase())
			        .AddSingleton<IUsers, UserRepository>()
			        .AddSingleton<IApiKeyValidator, ApiKeyValidator>()
			        .AddSingleton<IApiKeys, ApiKeyRepository>();

			services.AddAuthentication(options => {
				options.DefaultAuthenticateScheme = ApiKeyAuthenticationOptions.DefaultScheme;
				options.DefaultChallengeScheme = ApiKeyAuthenticationOptions.DefaultScheme;
			})
			.AddApiKeySupport(options=>{});
		}

		private IMongoDatabase SetupMongoDatabase() {
			var client = new MongoClient(Configuration.GetConnectionString("database"));
			return client.GetDatabase("robware");
		}

		// This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
		public void Configure(IApplicationBuilder app, IWebHostEnvironment env) {
			if (env.IsDevelopment()) {
				app.UseDeveloperExceptionPage();
			}

			app.UseHttpsRedirection();

			app.UseRouting();

			app.UseAuthentication();

			app.UseAuthorization();

			app.UseEndpoints(endpoints => {
				endpoints.MapControllers().RequireAuthorization();
			});
		}
	}
}
