﻿using MongoDB.Driver;
using Robware.Auth.Users;
using Robware.Data.Users;
using Robware.Data.Users.States;
using System.Threading.Tasks;

namespace Robware.Data {
	public class UserRepository : IUsers {
		private readonly IMongoCollection<UserState> _collection;

		public UserRepository(IMongoDatabase database) {
			_collection = database.GetCollection<UserState>("users");
		}

		public async Task<User> GetByEmail(string email) {
			var result = (await _collection.FindAsync(user => user.Email == email)).FirstOrDefault();
			if (result == null)
				throw new UserNotFoundException(email);

			return new DatabaseUser(result);
		}

		public async Task<bool> UpdateUserPassword(string email, string newPassword) {
			var result = (await _collection.UpdateOneAsync(state => state.Email == email, Builders<UserState>.Update.Set(state => state.Password, newPassword)));
			return result.ModifiedCount == 1;
		}
	}
}
